import javax.swing.Timer;
import java.awt.event.*;
import java.io.StringReader;
import java.io.EOFException;
import java.io.IOException;

/**
This class models the data link layer.
*/
public class DataLinkLayer
{
   /**
   Constructs a data link layer object.
   @param name the machine name
   @param packetStream the data
   @param error_rate the probability that a frame to this machine will have an error
   @param timeout how long this layer will wait before resend a frame
   @param delay how long in milliseconds it takes before an Event is processed
   */
   public DataLinkLayer(String name, String packetStream, double error_rate, 
      int timeout, int delay)
   {
      machineName = name;
      this.packetStream = new StringReader(packetStream);
      this.errorRate = error_rate;
      this.timeout = timeout;
      this.delay = delay;
      media = null;
      currentEvent = EventType.NO_EVENT;
      timeoutTimer = new Timer(timeout, 
         new ActionListener()
         {
            public void actionPerformed(ActionEvent evt)
            {
               setEventType(EventType.TIME_OUT);
               //~ timeoutTimer.stop();
            }
         });
      timeoutTimer.setRepeats(false);
   }
   
   public void setPhysicalLayer(PhysicalLayer p)
   {
      media = p;
   }
   
   public void setEventType(EventType e)
   {
      currentEvent = e;
   }
   
   public void protocol4()
   {
      new Timer(delay, 
         new ActionListener()
         {
            public ActionListener init()
            {
               buffer = fromNetworkLayer();
               Frame s = new Frame(nextFrameToSend, inc(frameExpected), buffer);
               toPhysicalLayer(s);
               startTimer();
               return this;
            }
            
            public void actionPerformed(ActionEvent evt)
            {
               EventType event = waitForEvent();
               if (event == EventType.FRAME_ARRIVAL)
               {
                  Frame r = fromPhysicalLayer();
                  boolean badFrame = true;
                  if (r.getSeq() == frameExpected)
                  {
                     toNetworkLayer(r.getInfo());
                     frameExpected = inc(frameExpected);
                     badFrame = false;
                  }
                  if (r.getAck() == nextFrameToSend)
                  {
                     stopTimer();
                     buffer = fromNetworkLayer();
                     nextFrameToSend = inc(nextFrameToSend);
                     badFrame = false;
                  }
                  if (badFrame)
                  {
                     event = EventType.CKSUM_ERR;
                  }
               }
               if (event == EventType.FRAME_ARRIVAL || event == EventType.TIME_OUT)
               {
                  Frame s = new Frame(nextFrameToSend, inc(frameExpected), buffer);
                  toPhysicalLayer(s);
                  startTimer();
               }
            }
            
            private int nextFrameToSend = 0;
            private int frameExpected = 0;
            private char buffer;
         }.init()).start();
   }
   
   public void toNetworkLayer(char packet)
   {
      System.out.print(machineName + "'s ");
      System.out.println("Network Layer Received: '" + packet + "'");
   }
   
   public char fromNetworkLayer()
   {
      int c;
      try
      {
         c = packetStream.read();
         if (c == -1)
            c = '\0';
      }
      catch (IOException e)
      {
         c = '\0';
      }
      return (char) c;
   }
   
   public void toPhysicalLayer(Frame s)
   {
      media.send(s, this);
      {
         System.out.print(machineName + "'s ");
         System.out.println("DataLink Layer sent frame: " + s);
      }
   }
   
   public Frame fromPhysicalLayer()
   {
      Frame f = media.getFrame();
      {
         if (Math.random() < errorRate)
            f.invalidate();
         System.out.print(machineName + "'s ");
         System.out.println("DataLink Layer Received frame: " + f);
      }
      return f;
   }
   
   public void startTimer()
   {
      timeoutTimer.restart();
   }
   
   public void stopTimer()
   {
      timeoutTimer.stop();
   }
   
   public int inc(int num)
   {
      if (num == 0)
         return 1;
      else if (num == 1)
         return 0;
      else
         return num;
   }
   
   public EventType waitForEvent()
   {
      EventType event = currentEvent;
      currentEvent = EventType.NO_EVENT;
      return event;
   }
   
   public String machineName;
   private StringReader packetStream;
   private double errorRate;
   private int timeout;
   private int delay;
   private PhysicalLayer media;
   private EventType currentEvent;
   private Timer timeoutTimer;
}
