/**
This class models the physical link layer.
*/
public class PhysicalLayer
{
   /**
   Constructs a physical link layer object.
   @param machine1 the first machine
   @param machine2 the second machine
   */
   public PhysicalLayer(DataLinkLayer machine1, DataLinkLayer machine2)
   {
      this.machine1 = machine1;
      this.machine2 = machine2;
      currentFrame = null;
      currentSender = null;
      machine1.setPhysicalLayer(this);
      machine2.setPhysicalLayer(this);
   }
   
   /**
   Sends one frame across the physical layer.
   @param frame the frame to send
   @param m the data link layer source
   */
   public void send(Frame frame, DataLinkLayer m)
   {
      /* If the physical layer is busy, the subsequent frame is lost. */
      if (currentSender != null && currentSender != m)
         return;
      currentSender = m;
      if (m == machine1)
      {
         machine2.setEventType(EventType.FRAME_ARRIVAL);
      }
      else if (m == machine2)
      {
         machine1.setEventType(EventType.FRAME_ARRIVAL);
      }
      currentFrame = frame;
   }
   
   /**
   Returns the frame in this physical layer.
   @return the frame
   */
   public Frame getFrame()
   {
      currentSender = null;
      return currentFrame;
   }
   
   private DataLinkLayer machine1;
   private DataLinkLayer machine2;
   private Frame currentFrame;
   private DataLinkLayer currentSender;
}
